--- gdal_calc.py	2011-11-28 16:30:44.694946429 +0100
+++ gdal_calc_mod.py	2011-11-28 16:29:41.032531187 +0100
@@ -1,23 +1,23 @@
 #!/usr/bin/env python
 #******************************************************************************
-# 
+#
 #  Project:  GDAL
 #  Purpose:  Command line raster calculator for gdal supported files
 #  Author:   Chris Yesson, chris.yesson@ioz.ac.uk
-# 
+#
 #******************************************************************************
 #  Copyright (c) 2010, Chris Yesson <chris.yesson@ioz.ac.uk>
-# 
+#
 #  Permission is hereby granted, free of charge, to any person obtaining a
 #  copy of this software and associated documentation files (the "Software"),
 #  to deal in the Software without restriction, including without limitation
 #  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 #  and/or sell copies of the Software, and to permit persons to whom the
 #  Software is furnished to do so, subject to the following conditions:
-# 
+#
 #  The above copyright notice and this permission notice shall be included
 #  in all copies or substantial portions of the Software.
-# 
+#
 #  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 #  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 #  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
@@ -28,7 +28,11 @@
 #******************************************************************************
 
 ################################################################
-# Command line raster calculator for gdal supported files. Use any basic arithmetic supported by numpy arrays such as +-*\ along with logical operators such as >.  Note that all files must be the same dimensions, but no projection checking is performed.  Use gdal_calc.py --help for list of options.
+# Command line raster calculator for gdal supported files. Use any basic arithmetic
+# supported by numpy arrays such as +-*\ along with logical operators such as >.
+# All files must have the same projection, be snapped, have the same pixel
+# size (a check is performed with a given tollerance).
+# Use gdal_calc.py --help for list of options.
 
 # example 1 - add two files together
 # gdal_calc.py -A input1.tif -B input2.tif --outfile=result.tif --calc="A+B"
@@ -50,6 +54,7 @@
 from optparse import OptionParser
 import sys
 import os
+import math
 
 # create alphabetic list for storing input layers
 AlphaList=["A","B","C","D","E","F","G","H","I","J","K","L","M",
@@ -58,10 +63,12 @@
 # set up some default nodatavalues for each datatype
 DefaultNDVLookup={'Byte':255, 'UInt16':65535, 'Int16':-32767, 'UInt32':4294967293, 'Int32':-2147483647, 'Float32':1.175494351E-38, 'Float64':1.7976931348623158E+308}
 
+Tolfactor = 0.00001
+
 #3.402823466E+38
 
 ################################################################
-def doit(opts, args):
+def grid_calc(opts, args):
 
     if opts.debug:
         print("gdal_calc.py starting calculation %s" %(opts.calc))
@@ -70,52 +77,119 @@
     # fetch details of input layers
     ################################################################
 
-    # set up some lists to store data for each band
+    # set the list of input layers
+    myFileList=[]
+    for myI in AlphaList[0:len(sys.argv)-1]:
+        myFile = eval("opts.%s" %(myI))
+        myBand = eval("opts.%s_band" %(myI))
+        if myFile:
+            myInfo = []
+            myInfo.append(myI)
+            myInfo.append(myFile)
+            # check if we have asked for a specific band...
+            if myBand:
+                myInfo.append(myBand)
+            else:
+                myInfo.append(1)
+            myFileList.append(myInfo)
+
+    # set up the lists to store info for each layer
     myFiles=[]
-    myBands=[]
-    myAlphaList=[]
     myDataType=[]
     myDataTypeNum=[]
+    myPixels=[]
     myNDV=[]
-    DimensionsCheck=None
+    myProjections=[]
 
-    # loop through input files - checking dimensions
-    for i,myI in enumerate(AlphaList[0:len(sys.argv)-1]):
-        myF = eval("opts.%s" %(myI))
-        myBand = eval("opts.%s_band" %(myI))
-        if myF:
-            myFiles.append(gdal.Open(myF, gdal.GA_ReadOnly))
-            # check if we have asked for a specific band...
-            if myBand:
-                myBands.append(myBand)
+    # create the extent array
+    myExtents=numpy.zeros(len(myFileList)*4)
+    myExtents=myExtents.reshape(len(myFileList),4)
+
+    # loop through input files
+    for i,myF in enumerate(myFileList):
+        myFiles.append(gdal.Open(myF[1], gdal.GA_ReadOnly))
+        myDataType.append(gdal.GetDataTypeName(myFiles[i].GetRasterBand(myF[2]).DataType))
+        myDataTypeNum.append(myFiles[i].GetRasterBand(myF[2]).DataType)
+        myPixels.append([myFiles[i].GetGeoTransform()[1],myFiles[i].GetGeoTransform()[5],
+                         myFiles[i].GetGeoTransform()[2],myFiles[i].GetGeoTransform()[4]])
+        myNDV.append(myFiles[i].GetRasterBand(myF[2]).GetNoDataValue())
+        myExtents[i,0]=myFiles[i].GetGeoTransform()[0]
+        myExtents[i,1]=myFiles[i].GetGeoTransform()[0] + (myFiles[i].RasterXSize*myFiles[i].GetGeoTransform()[1])
+        myExtents[i,2]=myFiles[i].GetGeoTransform()[3] - abs(myFiles[i].RasterYSize*myFiles[i].GetGeoTransform()[5])
+        myExtents[i,3]=myFiles[i].GetGeoTransform()[3]
+        myProjections.append(Files[i].GetProjection())
+
+        # layers checks
+        if i>0:
+            # check layer projection
+            if myProjections[0]==myProjections[i]:
+                pass
+            else:
+                print("Error! Projection of file %s is different from other files (%s). Cannot proceed." %(myF[1], myF[0][1]))
+
+            # check the pixels sizes and rotation
+            if abs(myPixels[0][0]-myPixels[i][0])<(myPixels[0][0]*Tolfactor):
+                if abs(myPixels[0][1]-myPixels[i][1])<abs(myPixels[0][1]*Tolfactor):
+                    if myPixels[0][2]==myPixels[i][2]:
+                        if myPixels[0][3]==myPixels[i][3]:
+                            pass
             else:
-                myBands.append(1)
-            myAlphaList.append(myI)
-            myDataType.append(gdal.GetDataTypeName(myFiles[i].GetRasterBand(myBands[i]).DataType))
-            myDataTypeNum.append(myFiles[i].GetRasterBand(myBands[i]).DataType)
-            myNDV.append(myFiles[i].GetRasterBand(myBands[i]).GetNoDataValue())
-            # check that the dimensions of each layer are the same
-            if DimensionsCheck:
-                if DimensionsCheck!=[myFiles[i].RasterXSize, myFiles[i].RasterYSize]:
-                    print("Error! Dimensions of file %s (%i, %i) are different from other files (%i, %i).  Cannot proceed" % \
-                            (myF,myFiles[i].RasterXSize, myFiles[i].RasterYSize,DimensionsCheck[0],DimensionsCheck[1]))
-                    return
+                print("Error! Pixel size or ratation of file %s (%s) are different from other files (%s). Cannot proceed." % \
+                      (myF[1],myPixels[i], myPixels[0]))
+                return
+
+            # check snap of layer
+            if abs(math.floor((myExtents[0,0]-myExtents[i,0])/myPixels[0][0]+0.5)*myPixels[0][0]-(myExtents[0,0]-myExtents[i,0]))<(myPixels[0][0]/Tolfactor):
+                if abs(math.floor((myExtents[0,1]-myExtents[i,1])/myPixels[0][1]+0.5)*myPixels[0][1]-(myExtents[0,1]-myExtents[i,1]))<(myPixels[0][1]/Tolfactor):
+                    pass
             else:
-                DimensionsCheck=[myFiles[i].RasterXSize, myFiles[i].RasterYSize]
+                print("Error! Layer %s is not snapped with other files. Cannot proceed.") %(myF[1])
+                return
 
-            if opts.debug:
-                print("file %s: %s, dimensions: %s, %s, type: %s" %(myI,myF,DimensionsCheck[0],DimensionsCheck[1],myDataType[i]))
+        if opts.debug:
+            print("file %s: %s, band: %s, dimensions: %s, %s, type: %s" % \
+                  (myF[0], myF[1], myF[2],myFiles[i].RasterXSize, myFiles[i].RasterYSize, myDataType[i]))
+            print("extent: %s, %s, %s, %s" %(myExtents[i,0], myExtents[i,1], myExtents[i,2], myExtents[i,3]))
 
     ################################################################
     # set up output file
     ################################################################
 
+    # define the output extent
+    myOutXmin = max(myExtents[:,0])
+    myOutXmax = min(myExtents[:,1])
+    myOutYmin = max(myExtents[:,2])
+    myOutYmax = min(myExtents[:,3])
+
+    if opts.debug:
+        print("the output extent is: %s, %s, %s, %s" %(myOutXmin, myOutXmax, myOutYmin, myOutYmax))
+
+    # check overlap
+    if myOutXmin<myOutXmax:
+        if myOutYmin<myOutYmax:
+            pass
+    else:
+        print("Error! Files don't overlap. Cannot proceed.")
+        return
+
+    # defune output dimensions
+    myOutcols = int((myOutXmax - myOutXmin)/myPixels[0][0]+0.5)
+    myOutrows = int((myOutYmax - myOutYmin)/abs(myPixels[0][1])+0.5)
+    myOutDimensions = [myOutcols,myOutrows]
+
+    # define the offsets for every source
+    myOffsets = []
+    for e in range(0,len(myFileList)):
+        Xoffset = int(((myOutXmin-myExtents[e][0])/myPixels[0][0])+0.5)
+        Yoffset = int((myExtents[e][3]-myOutYmax)/abs(myPixels[0][1])+0.5)
+        myOffsets.append([Xoffset,Yoffset])
+
     # open output file exists
     if os.path.isfile(opts.outF) and not opts.overwrite:
         if opts.debug:
             print("Output file %s exists - filling in results into file" %(opts.outF))
         myOut=gdal.Open(opts.outF, gdal.GA_Update)
-        if [myOut.RasterXSize,myOut.RasterYSize] != DimensionsCheck:
+        if [myOut.RasterXSize,myOut.RasterYSize] != myOutDimensions:
             print("Error! Output exists, but is the wrong size.  Use the --overwrite option to automatically overwrite the existing file")
             return
         myOutB=myOut.GetRasterBand(1)
@@ -139,10 +213,11 @@
 
         # create file
         myOutDrv = gdal.GetDriverByName(opts.format)
-        myOut=myOutDrv.Create(opts.outF, DimensionsCheck[0], DimensionsCheck[1], 1, gdal.GetDataTypeByName(myOutType))
+        myOut=myOutDrv.Create(opts.outF, myOutDimensions[0], myOutDimensions[1], 1, gdal.GetDataTypeByName(myOutType))
 
-        # set output geo info based on first input layer
-        myOut.SetGeoTransform(myFiles[0].GetGeoTransform())
+        # set output geo info
+        myOut.SetGeoTransform([myOutXmin, myPixels[0][0], myPixels[0][2],
+                               myOutYmax, myPixels[0][3], myPixels[0][1]])
         myOut.SetProjection(myFiles[0].GetProjection())
 
         myOutB=myOut.GetRasterBand(1)
@@ -160,17 +235,17 @@
         print("output file: %s, dimensions: %s, %s, type: %s" %(opts.outF,myOut.RasterXSize,myOut.RasterYSize,gdal.GetDataTypeName(myOutB.DataType)))
 
     ################################################################
-    # find block size to chop grids into bite-sized chunks 
+    # find block size to chop grids into bite-sized chunks
     ################################################################
 
-    # use the block size of the first layer to read efficiently
-    myBlockSize=myFiles[0].GetRasterBand(myBands[0]).GetBlockSize();
+    # use the block size of the output layer to read efficiently
+    myBlockSize=myOut.GetRasterBand(1).GetBlockSize();
     # store these numbers in variables that may change later
     nXValid = myBlockSize[0]
     nYValid = myBlockSize[1]
     # find total x and y blocks to be read
-    nXBlocks = (int)((DimensionsCheck[0] + myBlockSize[0] - 1) / myBlockSize[0]);
-    nYBlocks = (int)((DimensionsCheck[1] + myBlockSize[1] - 1) / myBlockSize[1]);
+    nXBlocks = (int)((myOutDimensions[0] + myBlockSize[0] - 1) / myBlockSize[0]);
+    nYBlocks = (int)((myOutDimensions[1] + myBlockSize[1] - 1) / myBlockSize[1]);
     myBufSize = myBlockSize[0]*myBlockSize[1]
 
     if opts.debug:
@@ -180,7 +255,7 @@
     ProgressCt=-1
     ProgressMk=-1
     ProgressEnd=nXBlocks*nYBlocks
-    
+
     ################################################################
     # start looping through blocks of data
     ################################################################
@@ -191,7 +266,7 @@
         # in the rare (impossible?) case that the blocks don't fit perfectly
         # change the block size of the final piece
         if X==nXBlocks-1:
-             nXValid = DimensionsCheck[0] - X * myBlockSize[0]
+             nXValid = myOutDimensions[0] - X * myBlockSize[0]
              myBufSize = nXValid*nYValid
 
         # find X offset
@@ -214,7 +289,7 @@
 
             # change the block size of the final piece
             if Y==nYBlocks-1:
-                nYValid = DimensionsCheck[1] - Y * myBlockSize[1]
+                nYValid = myOutDimensions[1] - Y * myBlockSize[1]
                 myBufSize = nXValid*nYValid
 
             # find Y offset
@@ -225,18 +300,21 @@
             myNDVs.shape=(nYValid,nXValid)
 
             # fetch data for each input layer
-            for i,Alpha in enumerate(myAlphaList):
+            for i,Alpha in enumerate(myFileList):
+
+                if opts.debug:
+                    print("for layer %s: %s the offsets is X = %s Y = %s" %(Alpha[0], Alpha[1], myOffsets[i][0], myOffsets[i][1]))
 
                 # populate lettered arrays with values
-                myval=BandReadAsArray(myFiles[i].GetRasterBand(myBands[i]),
-                                      xoff=myX, yoff=myY,
+                myval=BandReadAsArray(myFiles[i].GetRasterBand(Alpha[2]),
+                                      xoff=myX+myOffsets[i][0], yoff=myY+myOffsets[i][1],
                                       win_xsize=nXValid, win_ysize=nYValid)
 
                 # fill in nodata values
                 myNDVs=1*numpy.logical_or(myNDVs==1, myval==myNDV[i])
 
                 # create an array of values for this block
-                exec("%s=myval" %Alpha)
+                exec("%s=myval" %Alpha[0])
                 myval=None
 
 
@@ -247,7 +325,7 @@
                 print("evaluation of calculation %s failed" %(opts.calc))
                 raise
 
-            # propogate nodata values 
+            # propogate nodata values
             # (set nodata cells to zero then add nodata value to these cells)
             myResult = ((1*(myNDVs==0))*myResult) + (myOutNDV*myNDVs)
 
@@ -290,7 +368,7 @@
         print("No calculation provided.  Nothing to do!")
         print(usage)
     else:
-        doit(opts, args)
+        grid_calc(opts, args)
 
 if __name__ == "__main__":
     main()
